#!/bin/bash
#

# need an in source build

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e19:<p372|p383b|p392>>|<e20:<p383b|p392>>|<c7:<p372|p383b|p392>> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}


# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=nucondb
origpkgver=v2_5_1
pkgver=${origpkgver}
ssibuildshims_version=v1_04_15
pkgtarfile=${package}-${origpkgver}.tar 

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${basequal} in
e15:p2714b)
  cc=gcc
  cxx=g++
  stdcxx="-std=c++14" ;;
e17:p2714b)
  cc=gcc
  cxx=g++
  stdcxx="-std=c++17" ;;
c2:p2714b)
  cc=clang
  cxx=clang++
  stdcxx="-std=c++17"
   ;;
e17:p2715a|e17:p372)
  cc=gcc
  cxx=g++
  stdcxx="-std=c++17" ;;
e19:p2715a|e19:p372|e19:p383b|e19:p392)
  cc=gcc
  cxx=g++
  stdcxx="-std=c++17 -Wno-catch-value" ;;
e20:p383b|e20:p392)
  cc=gcc
  cxx=g++
  stdcxx="-std=c++17 -Wno-catch-value" ;;
c[27]:p2715a|c[27]:p372|c7:p383b|c7:p392)
  cc=clang
  cxx=clang++
  stdcxx="-std=c++17"
   ;;
*) 
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
esac

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

cd ${pkgdir} 
tar xf ${tardir}/${pkgtarfile} || ssi_die "ERROR: failed to unwind ${pkgtarfile}"
rm -rf build || ssi_die "ERROR: cannot remove old build subdirectory"

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

if [ -z ${LIBWDA_DIR} ]
then
   echo "ERROR: failed to setup libwda"
   exit 1
else
   echo "LIBWDA_DIR: ${LIBWDA_DIR}"
   echo "LIBWDA_LIB: ${LIBWDA_LIB}"
fi
if [ -z ${IFDHC_DIR} ]
then
   echo "ERROR: failed to setup ifdhc"
   exit 1
else
   echo "IFDHC_DIR: ${IFDHC_DIR}"
   echo "IFDHC_LIB: ${IFDHC_LIB}"
fi

echo

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

case ${extraqual} in
debug) ARCH="${stdcxx} -O0 -g";;
prof)  ARCH="${stdcxx} -O3 -g -DNDEBUG ";;
esac
export ARCH

set -x

cd ${pkgdir}/src

env CC=${cc} CXX=${cxx} make all || ssi_die "ERROR: make all failed"
env CC=${cc} CXX=${cxx} make install || ssi_die "ERROR: make install failed"
env CC=${cc} CXX=${cxx} make clean || ssi_die "ERROR: make clean failed"

set +x

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi


# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "ERROR: setup failed"

# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
